@@ -1,14 +1,16 @@
 ;;; vimish-fold.el --- Fold text like in Vim -*- lexical-binding: t; -*-
 ;;
-;; Copyright © 2015–2017 Mark Karpov <markkarpov92@gmail.com>
+;; Copyright © 2020–present Sergey Matsievskiy <matsievskiysv@gmail.com>
+;; Copyright © 2015–2020 Mark Karpov <markkarpov92@gmail.com>
 ;; Copyright © 2012–2013 Magnar Sveen <magnars@gmail.com>
 ;;
+;; Author: Sergey Matsievskiy <matsievskiysv@gmail.com>
 ;; Author: Mark Karpov <markkarpov92@gmail.com>
 ;; Author: Magnar Sveen <magnars@gmail.com>
 ;; URL: https://github.com/mrkkrp/vimish-fold
-;; Package-Version: 0.2.3
-;; Package-Revision: e631352fbf91
-;; Package-Requires: ((emacs "24.4") (cl-lib "0.5") (f "0.18.0"))
+;; Package-Version: 20251023.1551
+;; Package-Revision: f71f374d28a8
+;; Package-Requires: ((emacs "26.1") (cl-lib "0.5") (f "0.18.0"))
 ;; Keywords: convenience
 ;;
 ;; This file is not part of GNU Emacs.
@@ -102,14 +104,15 @@ If set to NIL, do not indicate folded text, just highlight it."
   :tag  "Header of Blank Fold"
   :type 'string)
 
-(defcustom vimish-fold-header-width 80
+(defcustom vimish-fold-header-width nil
   "Width of header of folded region.
 
 This can be a number or NIL.  If it's NIL value returned of
 `window-width' will be used."
   :tag  "Width of header of folded region"
   :type '(choice (const   :tag "use window width")
-                 (integer :tag "width of fold header")))
+                 (integer :tag "width of fold header"))
+  :package-version '(vimish-fold . "0.4.0"))
 
 (defcustom vimish-fold-show-lines t
   "Whether to show number of lines folded in fold header."
@@ -138,27 +141,37 @@ overhead is undesirable."
 (defvar vimish-fold-unfolded-keymap (make-sparse-keymap)
   "Keymap which is active when point is placed on unfolded text.")
 
-(defun vimish-fold--correct-region (beg end)
-  "Return a cons of corrected BEG and END.
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; <<< Helper functions
 
-We only support folding by whole lines, so we should make sure
-that beginning and end positions are correct.  Also, sometimes
-users select region including last newline into it, they don't
-really want to include it, we correct this here."
-  (cl-destructuring-bind (beg . end)
-      (if (>= end beg)
-          (cons beg end)
-        (cons end beg))
-    (save-excursion
-      (let* ((beg* (progn (goto-char beg)
-                          (line-beginning-position)))
-             (end* (progn (goto-char end)
-                          (if (and (zerop (current-column))
-                                   (/= end beg*)
-                                   (not vimish-fold-include-last-empty-line))
-                              (1- end)
-                            (line-end-position)))))
-        (cons beg* end*)))))
+(defun vimish-fold--vimish-overlay-folded-p (overlay)
+  "Detect if given OVERLAY is folded."
+  (eq (overlay-get overlay 'type) 'vimish-fold--folded))
+
+(defun vimish-fold--vimish-overlay-unfolded-p (overlay)
+  "Detect if given OVERLAY is folded."
+  (eq (overlay-get overlay 'type) 'vimish-fold--unfolded))
+
+(defun vimish-fold--vimish-overlay-p (overlay)
+  "Detect if given OVERLAY is created by this package."
+  (memq (overlay-get overlay 'type)
+        '(vimish-fold--folded
+          vimish-fold--unfolded)))
+
+(cl-defun vimish-fold--folds-in (beg end &optional (type 'all))
+  "Return all folds exiting between BEG and END in current buffer.
+
+Set TYPE to folded or unfolded to return only folded or unfolded
+overlays.
+When TYPE is omitted function returns all fold overlays."
+  (overlay-recenter end)               ; speedup overlay lookup near end
+  (cl-remove-if-not
+   (cl-case type
+     ('folded #'vimish-fold--vimish-overlay-folded-p)
+     ('unfolded #'vimish-fold--vimish-overlay-unfolded-p)
+     (t #'vimish-fold--vimish-overlay-p))
+   (overlays-in beg end)))
 
 (defun vimish-fold--read-only (on beg end)
   "If ON is non-NIL, make text between BEG and END read-only.
@@ -166,32 +179,80 @@ really want to include it, we correct this here."
 If ON is NIL, make the text editable again."
   (let ((inhibit-read-only t))
     (with-silent-modifications
-      (funcall
-       (if on #'add-text-properties #'remove-text-properties)
-       beg end (list 'read-only on)))))
+      (if on
+          (add-text-properties beg end (list 'read-only on))
+        (remove-text-properties beg end (list 'read-only on))))))
 
-(defun vimish-fold--get-header (beg end)
+(defun vimish-fold--correct-region (beg end)
+  "Return a cons of corrected BEG and END.
+
+We only support folding by whole lines, so we should make sure
+that beginning and end positions are correct.  Also, sometimes
+users select region including last newline into it, they don't
+really want to include it, we correct this here."
+  (cl-destructuring-bind (beg . end)    ; sort BEG and END
+      (if (>= end beg)
+          (cons beg end)
+        (cons end beg))
+    (save-excursion
+      (save-restriction
+        (let ((beg* (progn (goto-char beg) ; Expand BEG and END to whole lines
+                           (line-beginning-position)))
+              (end* (progn (goto-char end)
+                           (line-end-position))))
+          (when (or vimish-fold-include-last-empty-line
+                    (not (eq beg* end*)))
+            (narrow-to-region beg* end*)
+            (goto-char (point-min))
+            ;; Iterate over strings in region and find empty ones
+            (let* ((empty-lines (reverse (cl-loop if (progn (goto-char (line-beginning-position)) (looking-at "$"))
+                                                  collect (line-number-at-pos)
+                                                  end
+                                                  until (progn (goto-char (line-end-position)) (eobp))
+                                                  do (forward-line 1))))
+                   (empty-lines-num (length empty-lines)))
+              ;; Remove last line only if it is preceded by non-empty line
+              ;; https://github.com/seregaxvm/vimish-fold/issues/55
+              (cl-case empty-lines-num
+                (0)
+                (1 (when (eq (line-number-at-pos (point-max)) (car empty-lines))
+                     (setq end* (1- end*))))
+                (t (when (and (eq (line-number-at-pos (point-max)) (car empty-lines))
+                              (> (- (car empty-lines) (cadr empty-lines)) 1))
+                     (setq end* (1- end*)))))))
+          (cons beg* end*))))))
+
+(defun vimish-fold--get-header (beg end &optional buffer-or-name)
   "Extract folding header from region between BEG and END in BUFFER.
 
-If BUFFER is NIL, current buffer is used."
-  (let ((info (when vimish-fold-show-lines
-                (format "    %d lines" (count-lines beg end)))))
-    (save-excursion
-      (goto-char beg)
-      (re-search-forward "^\\([[:blank:]]*.+\\)$")
-      (concat
-       (truncate-string-to-width
-        (if (and (>= (match-beginning 1) beg)
-                 (<= (match-end 1)       end))
-            (match-string-no-properties 1)
-          vimish-fold-blank-fold-header)
-        (- (or vimish-fold-header-width
-               (window-width))
-           (length info))
-        nil
-        32 ; space
-        "…")
-       info))))
+If BUFFER-OR-NAME is nil, current buffer is used."
+  (with-current-buffer (or buffer-or-name (current-buffer))
+    (let ((info (when vimish-fold-show-lines
+                  (format "    %d lines" (count-lines beg end)))))
+      (save-excursion
+        (goto-char beg)
+        (re-search-forward "^\\([[:blank:]]*.*\\)$")
+        (concat
+         (truncate-string-to-width
+          (if (and (>= (match-beginning 1) beg)
+                   (<= (match-end 1)       end))
+              (match-string-no-properties 1)
+            vimish-fold-blank-fold-header)
+          ;; https://debbugs.gnu.org/cgi/bugreport.cgi?bug=41537
+          ;; when applying folds on newly created buffer,
+          ;; `line-number-display-width' fails because window is not ready yet.
+          ;; In this case assume 5 line number chars + 2 padding chars = 7 chars
+          (- (or vimish-fold-header-width
+                 (- (window-body-width)
+                    (if (and (get-buffer-window (current-buffer))
+                               line-number-mode)
+                        (+ (line-number-display-width) 2)
+                      7)))
+             (length info))
+          nil
+          32 ; space
+          "…")
+         info)))))
 
 (defun vimish-fold--setup-fringe (overlay &optional prefix)
   "Setup fringe for OVERLAY according to user settings.
@@ -219,11 +280,27 @@ This includes fringe bitmaps and faces."
   (overlay-put overlay 'help-echo "Click to unfold the text")
   (vimish-fold--setup-fringe overlay))
 
-(defun vimish-fold--vimish-overlay-p (overlay)
-  "Detect if given OVERLAY is created by this package."
-  (memq (overlay-get overlay 'type)
-        '(vimish-fold--folded
-          vimish-fold--unfolded)))
+(defun vimish-fold--update-headers (frame)
+  "Update headers"
+  (dolist (buffer (buffer-list frame))
+    (let ((window (get-buffer-window)))
+      (when (and (not (eq (window-pixel-width-before-size-change window)
+                          (window-pixel-height window))))
+        ;; run for all buffers with changed width
+        (with-current-buffer buffer
+          (dolist (overlay (vimish-fold--folds-in (point-min)
+                                                  (point-max)
+                                                  'folded))
+            (vimish-fold--apply-cosmetic overlay
+                                         (vimish-fold--get-header
+                                          (overlay-start overlay)
+                                          (overlay-end overlay)))))))))
+
+;; >>>
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; <<< Core functions
 
 ;;;###autoload
 (defun vimish-fold (beg end)
@@ -233,10 +310,9 @@ This includes fringe bitmaps and faces."
   (cl-destructuring-bind (beg . end) (vimish-fold--correct-region beg end)
     (when (< (count-lines beg end) 2)
       (error "Nothing to fold"))
-    (dolist (overlay (overlays-in beg end))
-      (when (vimish-fold--vimish-overlay-p overlay)
+    (dolist (overlay (vimish-fold--folds-in beg end))
         (goto-char (overlay-start overlay))
-        (error "Fold already exists here")))
+        (error "Fold already exists here"))
     (vimish-fold--read-only t (max 1 (1- beg)) end)
     (let ((overlay (make-overlay beg end nil t nil)))
       (overlay-put overlay 'type 'vimish-fold--folded)
@@ -247,7 +323,7 @@ This includes fringe bitmaps and faces."
 
 (defun vimish-fold--unfold (overlay)
   "Unfold fold found by its OVERLAY type `vimish-fold--folded'."
-  (when (eq (overlay-get overlay 'type) 'vimish-fold--folded)
+  (when (vimish-fold--vimish-overlay-folded-p overlay)
     (let ((beg (overlay-start overlay))
           (end (overlay-end   overlay)))
       (vimish-fold--read-only nil (max 1 (1- beg)) end)
@@ -256,7 +332,8 @@ This includes fringe bitmaps and faces."
         (overlay-put unfolded 'type 'vimish-fold--unfolded)
         (overlay-put unfolded 'evaporate t)
         (overlay-put unfolded 'keymap vimish-fold-unfolded-keymap)
-        (vimish-fold--setup-fringe unfolded t)))))
+        (vimish-fold--setup-fringe unfolded t)
+        unfolded))))
 
 ;;;###autoload
 (defun vimish-fold-unfold ()
@@ -269,9 +346,11 @@ This includes fringe bitmaps and faces."
 
 (defun vimish-fold--refold (overlay)
   "Refold fold found by its OVERLAY type `vimish-fold--unfolded'."
-  (when (eq (overlay-get overlay 'type) 'vimish-fold--unfolded)
-    (let* ((beg (overlay-start overlay))
-           (end (overlay-end   overlay)))
+  (when (vimish-fold--vimish-overlay-unfolded-p overlay)
+    (when (buffer-narrowed-p)
+      (error "Cannot operate on narrowed buffers"))
+    (let ((beg (overlay-start overlay))
+          (end (overlay-end   overlay)))
       (delete-overlay overlay)
       (vimish-fold beg end))))
 
@@ -288,8 +367,7 @@ This includes fringe bitmaps and faces."
 
 If OVERLAY does not represent a fold, it's ignored."
   (when (vimish-fold--vimish-overlay-p overlay)
-    (when (eq (overlay-get overlay 'type)
-              'vimish-fold--folded)
+    (when (vimish-fold--vimish-overlay-folded-p overlay)
       (vimish-fold--read-only
        nil
        (max 1 (1- (overlay-start overlay)))
@@ -302,24 +380,27 @@ If OVERLAY does not represent a fold, it's ignored."
   (interactive)
   (mapc #'vimish-fold--delete (overlays-at (point))))
 
+;; >>>
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Extra features
-
-(defun vimish-fold--folds-in (beg end)
-  "Return all folds exiting between BEG and END in current buffer."
-  (cl-remove-if-not
-   #'vimish-fold--vimish-overlay-p
-   (overlays-in beg end)))
+;; <<< Extra features
 
 ;;;###autoload
 (defun vimish-fold-unfold-all ()
   "Unfold all folds in current buffer."
   (interactive)
   (mapc #'vimish-fold--unfold
-        (vimish-fold--folds-in
-         (point-min)
-         (point-max))))
+        (vimish-fold--folds-in (point-min) (point-max))))
+
+;;;###autoload
+(defun vimish-fold-unfold-all-others ()
+  "Unfold all but current folds in current buffer."
+  (interactive)
+  (mapc #'vimish-fold--unfold
+        (cl-nset-difference
+         (vimish-fold--folds-in (point-min) (point-max))
+         (overlays-at (point)))))
 
 ;;;###autoload
 (defun vimish-fold-refold-all ()
@@ -327,26 +408,40 @@ If OVERLAY does not represent a fold, it's ignored."
   (interactive)
   (save-excursion ; after folding cursor jumps to beginning of fold
     (mapc #'vimish-fold--refold
-          (vimish-fold--folds-in
-           (point-min)
-           (point-max)))))
+          (vimish-fold--folds-in (point-min) (point-max)))))
+
+;;;###autoload
+(defun vimish-fold-refold-all-others ()
+  "Refold all closed folds but current in current buffer."
+  (interactive)
+  (save-excursion ; after folding cursor jumps to beginning of fold
+    (mapc #'vimish-fold--refold
+          (cl-nset-difference
+           (vimish-fold--folds-in (point-min) (point-max))
+           (overlays-at (point))))))
 
 ;;;###autoload
 (defun vimish-fold-delete-all ()
   "Delete all folds in current buffer."
   (interactive)
   (mapc #'vimish-fold--delete
-        (vimish-fold--folds-in
-         (point-min)
-         (point-max))))
+        (vimish-fold--folds-in (point-min) (point-max))))
+
+;;;###autoload
+(defun vimish-fold-delete-all-others ()
+  "Delete all folds but current in current buffer."
+  (interactive)
+  (mapc #'vimish-fold--delete
+        (cl-nset-difference
+         (vimish-fold--folds-in (point-min) (point-max))
+         (overlays-at (point)))))
 
 (defun vimish-fold--toggle (overlay)
   "Unfold or refold fold represented by OVERLAY depending on its type."
   (when (vimish-fold--vimish-overlay-p overlay)
     (save-excursion
       (goto-char (overlay-start overlay))
-      (if (eq (overlay-get overlay 'type)
-              'vimish-fold--folded)
+      (if (vimish-fold--vimish-overlay-folded-p overlay)
           (vimish-fold-unfold)
         (vimish-fold-refold)))))
 
@@ -365,6 +460,15 @@ If OVERLAY does not represent a fold, it's ignored."
          (point-min)
          (point-max))))
 
+;;;###autoload
+(defun vimish-fold-toggle-all-others ()
+  "Toggle all folds but current in current buffer."
+  (interactive)
+  (mapc #'vimish-fold--toggle
+        (cl-nset-difference
+         (vimish-fold--folds-in (point-min) (point-max))
+         (overlays-at (point)))))
+
 (declare-function avy-goto-line "ext:avy")
 
 ;;;###autoload
@@ -416,9 +520,103 @@ This feature needs `avy' package."
            folds-before-point)))
       (message "No more folds before point"))))
 
+;;;###autoload
+(defun vimish-fold-narrow-to-fold ()
+  "Narrow to fold at current position."
+  (interactive)
+  (cl-loop for overlay in (overlays-at (point))
+           if (vimish-fold--vimish-overlay-p overlay)
+           do
+           (when (vimish-fold--vimish-overlay-folded-p overlay)
+             (setq overlay (vimish-fold--unfold overlay)))
+           (narrow-to-region (overlay-start overlay)
+                             (overlay-end overlay))
+           end
+           until (vimish-fold--vimish-overlay-p overlay)))
+
+(define-key vimish-fold-folded-keymap (kbd "C-x n f") #'vimish-fold-narrow-to-fold)
+(define-key vimish-fold-unfolded-keymap (kbd "C-x n f") #'vimish-fold-narrow-to-fold)
+
+
+;; >>>
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Persistence
+;; <<< Folding marks
+
+(defcustom vimish-fold-marks '("{{{" . "}}}")
+  "Strings marking beginning and end of folding region.
+
+Automatically create folds from regions marked by `vimish-fold-marks' strings."
+  :tag "Fold marks."
+  :type '(cons :tag "Configure marks" string string)
+  :package-version '(vimish-fold . "0.3.0")
+  :safe #'vimish-fold--check-marks)
+(make-variable-buffer-local 'vimish-fold-marks)
+
+(defcustom vimish-fold-find-marks-on-open t
+  "Whether to search for marks when opening buffer."
+  :tag "Search for marks on open."
+  :type 'boolean
+  :package-version '(vimish-fold . "0.3.0"))
+
+(defun vimish-fold--check-marks (arg)
+  "Check that ARG is a valid `vimish-fold-marks' value."
+  (and (listp arg)
+       (stringp (car arg))
+       (stringp (cdr arg))))
+
+(defun vimish-fold--find-marks-in-region (beg end)
+  "Find folding symbols in region between BEG and END.
+
+Returns list of regions between marks in form
+  (BEG END UNFOLDED)."
+  (save-excursion
+    (save-restriction
+      (narrow-to-region beg end)
+      (goto-char (point-min))
+      (let (start end)
+        (cl-loop
+         do
+         (setq start (search-forward (car vimish-fold-marks) nil t)
+               end (search-forward (cdr vimish-fold-marks) nil t))
+         while (and start end)
+         unless (= (line-number-at-pos start)
+                   (line-number-at-pos end))
+         collect (list start end 'UNFOLDED))))))
+
+(defun vimish-fold--find-gaps ()
+  "Find gaps between folds."
+  (let ((overlays (vimish-fold--folds-in (point-min) (point-max)))
+        positions)
+    (setq positions
+          (append (list (point-min))
+                  (cl-mapcan #'list
+                             (cl-sort (cl-mapcar 'overlay-start overlays) '<)
+                             (cl-sort (cl-mapcar 'overlay-end overlays) '<))
+                  (list (point-max))))
+    (cl-remove-if
+     (lambda (x) (let ((beg (line-number-at-pos (car x)))
+                  (end (line-number-at-pos (cadr x))))
+              (> 2 (- end beg))))
+     (cl-loop while positions
+              collect (list (pop positions) (pop positions))))))
+
+;;;###autoload
+(defun vimish-fold-from-marks ()
+  "Create folds from folding symbols.
+
+Mark strings are controlled by `vimish-fold-marks' customize variable."
+  (interactive)
+  (vimish-fold--restore-from
+   (cl-mapcan (lambda (x) (apply 'vimish-fold--find-marks-in-region x))
+              (vimish-fold--find-gaps))))
+
+;; >>>
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; <<< Persistence
 
 (defcustom vimish-fold-dir
   (file-name-as-directory (f-expand "vimish-fold" user-emacs-directory))
@@ -443,9 +641,9 @@ Elements of LIST should be of the following form:
 
   (BEG END &optional UNFOLDED)"
   (save-excursion
-    (dolist (item list)
+     (dolist (item list)
       (cl-destructuring-bind (beg end . rest) item
-        (funcall #'vimish-fold beg end)
+        (vimish-fold beg end)
         (when (car rest)
           (goto-char beg)
           (vimish-fold-unfold))))))
@@ -457,15 +655,14 @@ BUFFER-OR-NAME defaults to current buffer."
   (with-current-buffer (or buffer-or-name (current-buffer))
     (let ((filename (buffer-file-name))
           regions)
-      (when filename
+      (when (and vimish-fold-mode filename)
         (dolist (overlay (overlays-in (point-min) (point-max)))
           (when (vimish-fold--vimish-overlay-p overlay)
             (push (list (overlay-start overlay)
                         (overlay-end   overlay)
-                        (eq (overlay-get overlay 'type)
-                            'vimish-fold--unfolded))
+                        (vimish-fold--vimish-overlay-unfolded-p overlay))
                   regions)))
-        (let ((fold-file (vimish-fold--make-file-name filename)))
+        (let ((fold-file (vimish-fold--make-file-name (f-canonical filename))))
           (if regions
               (with-temp-buffer
                 (pp regions (current-buffer))
@@ -477,7 +674,9 @@ BUFFER-OR-NAME defaults to current buffer."
                         (message nil))
                     (file-error
                      (message "Vimish Fold: can't write %s" fold-file)))
-                  (kill-buffer (current-buffer))))
+                  (kill-buffer (current-buffer))
+                  (message "Saved folds... %s" fold-file)
+                  (message nil)))
             (when (f-exists? fold-file)
               (f-delete fold-file))))))))
 
@@ -493,17 +692,25 @@ Return T is some folds have been restored and NIL otherwise."
                  (null (vimish-fold--folds-in
                         (point-min)
                         (point-max))))
-        (let ((fold-file (vimish-fold--make-file-name filename)))
+        (let ((fold-file (vimish-fold--make-file-name (f-canonical filename))))
           (when (and fold-file (f-readable? fold-file))
+            (message "Restoring folds... %s" (buffer-file-name))
+            (message nil)
             (vimish-fold--restore-from
              (with-temp-buffer
                (insert-file-contents fold-file)
                (read (buffer-string))))))))))
 
-(defun vimish-fold--kill-emacs-hook ()
+(defun vimish-fold--kill-emacs ()
   "Traverse all buffers and try to save their folds."
   (mapc #'vimish-fold--save-folds (buffer-list)))
 
+;; >>>
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; <<< Mode definition
+
 ;;;###autoload
 (define-minor-mode vimish-fold-mode
   "Toggle `vimish-fold-mode' minor mode.
@@ -517,19 +724,32 @@ This minor mode sets hooks so when you `find-file' it calls
 `vimish-fold--restore-folds' and when you kill a file it calls
 `vimish-fold--save-folds'.
 
-For globalized version of this mode see `vimish-gold-global-mode'."
+For globalized version of this mode see `vimish-fold-global-mode'."
   :global nil
-  (let ((fnc (if vimish-fold-mode #'add-hook #'remove-hook)))
-    (funcall fnc 'find-file-hook   #'vimish-fold--restore-folds)
-    (funcall fnc 'kill-buffer-hook #'vimish-fold--save-folds)
-    (funcall fnc 'kill-emacs-hook  #'vimish-fold--kill-emacs-hook)
-    (when vimish-fold-persist-on-saving
-      (funcall fnc 'before-save-hook #'vimish-fold--save-folds))))
+  (when (buffer-file-name)
+    (if vimish-fold-mode
+        (catch 'restore (vimish-fold--restore-folds)
+          (when vimish-fold-find-marks-on-open
+            (vimish-fold-from-marks)))
+      (vimish-fold-delete-all))))
 
 ;;;###autoload
 (define-globalized-minor-mode vimish-fold-global-mode
   vimish-fold-mode vimish-fold-mode)
 
+(add-hook 'kill-buffer-hook #'vimish-fold--save-folds)
+(when vimish-fold-persist-on-saving
+  (add-hook 'before-save-hook #'vimish-fold--save-folds))
+(add-hook 'kill-emacs-hook  #'vimish-fold--kill-emacs)
+(unless vimish-fold-header-width
+  (add-hook 'window-size-change-functions
+            #'vimish-fold--update-headers))
+
+;; >>>
+
 (provide 'vimish-fold)
 
+;; Local Variables:
+;; vimish-fold-marks: ("<<<" . ">>>")
+;; End:
 ;;; vimish-fold.el ends here
